# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'horizontalslider.ui'
#
# Created by: PyQt5 UI code generator 5.13.0
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(400, 300)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(Form.sizePolicy().hasHeightForWidth())
        Form.setSizePolicy(sizePolicy)
        self.verticalLayout = QtWidgets.QVBoxLayout(Form)
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout.setObjectName("verticalLayout")
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setContentsMargins(0, 0, 0, 0)
        self.gridLayout.setObjectName("gridLayout")
        self.value = QtWidgets.QSlider(Form)
        self.value.setOrientation(QtCore.Qt.Horizontal)
        self.value.setTickPosition(QtWidgets.QSlider.TicksAbove)
        self.value.setObjectName("value")
        self.gridLayout.addWidget(self.value, 1, 0, 1, 1)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.min = QtWidgets.QLabel(Form)
        self.min.setObjectName("min")
        self.horizontalLayout.addWidget(self.min)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.max = QtWidgets.QLabel(Form)
        self.max.setObjectName("max")
        self.horizontalLayout.addWidget(self.max)
        self.gridLayout.addLayout(self.horizontalLayout, 0, 0, 1, 1)
        self.units = QtWidgets.QLabel(Form)
        self.units.setObjectName("units")
        self.gridLayout.addWidget(self.units, 1, 1, 1, 1)
        self.verticalLayout.addLayout(self.gridLayout)
        self.label = QtWidgets.QLabel(Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label.sizePolicy().hasHeightForWidth())
        self.label.setSizePolicy(sizePolicy)
        self.label.setAlignment(QtCore.Qt.AlignHCenter|QtCore.Qt.AlignTop)
        self.label.setObjectName("label")
        self.verticalLayout.addWidget(self.label)
        spacerItem1 = QtWidgets.QSpacerItem(0, 0, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem1)

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Form"))
        self.min.setText(_translate("Form", "TextLabel"))
        self.max.setText(_translate("Form", "TextLabel"))
        self.units.setText(_translate("Form", "TextLabel"))
        self.label.setText(_translate("Form", "TextLabel"))



#!/usr/bin/env python3

#TODO: """DocString if there is one"""

import epyqlib.abstractpluginclass
import os
from PyQt5.QtCore import QFileInfo

# See file COPYING in this source tree
__copyright__ = 'Copyright 2016, EPC Power Corp.'
__license__ = 'GPLv2+'


class AbstractSignalPlugin(epyqlib.abstractpluginclass.AbstractPlugin):
    def __init__(self, parent=None, in_designer=True):
        epyqlib.abstractpluginclass.AbstractPlugin.__init__(
            self,
            parent=parent,
            in_designer=in_designer
        )

        self.in_designer = in_designer

    def createWidget(self, parent):
        return self._init(parent, in_designer=True)


if __name__ == '__main__':
    import sys

    print('No script functionality here')
    sys.exit(1)     # non-zero is a failure



import threading

import psutil


def spy(*ignore):
    """
    Make sure to ignore callables that are more than just their __call__
    because this will destroy everything else.
    """
    def inner(cls):
        class Spy(cls):
            def __getattribute__(self, name):
                attribute = super().__getattribute__(name)
                if hasattr(attribute, '__call__') and name not in ignore:

                    def mark(self, marker):
                        print('{marker} '
                              '>Thread Id 0x{thread_id:012x}< '
                              '{repr} '
                              '{marker} '
                              '{name}()'.format(
                                marker=marker,
                                repr='<{cls} object at 0x{id:012x}>'.format(
                                    cls=type(self).__bases__[0].__name__,
                                    id=id(self)
                                ),
                                name=name,
                                thread_id=threading.get_ident()
                        ))

                    def report(*args, **kwargs):
                        mark(self, ' +')
                        result = attribute(*args, **kwargs)
                        mark(self, '- ')

                        return result

                    return report
                else:
                    return attribute

        return Spy

    return inner


class ReportThreads:
    def __init__(self):
        self.process = psutil.Process()
        self.i = 0
        self.previous_threads = set()

    def __call__(self, description=''):
        current_threads = {t.id for t in self.process.threads()}

        started_threads = current_threads - self.previous_threads
        stopped_threads = self.previous_threads - current_threads

        print('report_threads()', str(self.i).rjust(5), description)
        print('    ', threading.enumerate())

        x = (
            ('current', current_threads),
            ('started', started_threads),
            ('stopped', stopped_threads),
        )

        for d, v in x:
            print('    ', d, sorted(v))

        self.i += 1
        self.previous_threads = current_threads

report_threads = ReportThreads()



import collections
import io
import math
import textwrap

import attr

import epyqlib.utils.general
import epyqlib.canneo

# See file COPYING in this source tree
__copyright__ = 'Copyright 2017, EPC Power Corp.'
__license__ = 'GPLv2+'


class MessageType(epyqlib.utils.general.AutoNumberIntEnum):
    Rx = ()
    Tx = ()
    Error = ()


@attr.s(hash=True)
class Log(epyqlib.canneo.QtCanListener):
    name = attr.ib()
    messages = attr.ib(default=None, hash=False)
    _active = attr.ib(default=False)
    _messages_factory = attr.ib(
        default=lambda: collections.deque(maxlen=100000)
    )

    def __attrs_post_init__(self):
        super().__init__(receiver=self._message_received)

        self.clear()

    def _message_received(self, message):
        if self._active:
            self.messages.append(Message.from_pythoncan(message))

    def start(self):
        self._active = True

    def stop(self):
        self._active = False

    def clear(self):
        self.messages = self._messages_factory()

    def restart(self):
        self.clear()
        self.start()

    def minimum_timestamp(self):
        timestamps = (m.time for m in self.messages if m.time is not None)

        for t in timestamps:
            return t

        return None


@attr.s
class Id:
    value = attr.ib()
    extended = attr.ib()


@attr.s
class Message:
    time = attr.ib()
    type = attr.ib()
    id = attr.ib()
    data = attr.ib()

    @classmethod
    def from_pythoncan(cls, message):
        return cls(
            time=message.timestamp,
            type=MessageType.Rx,
            id=Id(
                value=message.arbitration_id,
                extended=message.id_type,
            ),
            data=bytearray(message.data)
        )

    @property
    def ms(self):
        if self.time is None:
            return self.time

        return self.time * 1000

    @ms.setter
    def ms(self, value):
        if value is not None:
            value = value / 1000

        self.time = value

    @property
    def data_string_spaced(self):
        return ' '.join('{:02X}'.format(b) for b in self.data)

    @property
    def length(self):
        return len(self.data)


def to_trc_v1_1_s(messages):
    s = io.StringIO()

    to_trc_v1_1(messages, s)

    s.seek(0)

    return s.read()


def to_trc_v1_1(messages, f):
    header = textwrap.dedent('''\
        ;$FILEVERSION=1.1
        ;$STARTTIME={start_time}
        ;
        ;   {path}
        ;
        ;   Start time: {start_string}
        ;   Generated by EPyQ {version_string}
        ;
        ;   Message Number
        ;   |         Time Offset (ms)
        ;   |         |        Type
        ;   |         |        |        ID (hex)
        ;   |         |        |        |     Data Length
        ;   |         |        |        |     |   Data Bytes (hex) ...
        ;   |         |        |        |     |   |
        ;---+--   ----+----  --+--  ----+---  +  -+ -- -- -- -- -- -- --'''
    ).format(
        start_time=0,
        path=f.name,
        start_string='',
        version_string=''
    )

    format = '  '.join((
        '{i: 6d})',
        '{ms: 10.1f}',
        '{type:<5s}',
        '{id:08X}',
        '{length:1d}',
        '{data}',
    ))
    format += ' \n'

    for line in header.splitlines():
        f.write(line.rstrip() + '\n')

    for i, message in enumerate(messages, start=1):
        f.write(format.format(
            i=i,
            ms=message.ms,
            type=message.type.name,
            id=message.id.value,
            length=message.length,
            data=message.data_string_spaced,
        ))

def to_trc_v1_3(messages, bus):
    """`messages` should be a dict.  Keys are bus numbers and values are
    iterables of messages"""
    raise Exception('Not implemented')



#!/usr/bin/env python3

#TODO: """DocString if there is one"""

import enum
import struct
import socket

from ctypes import *


# See file COPYING in this source tree
__copyright__ = 'Copyright 2016, EPC Power Corp.'
__license__ = 'GPLv2+'


# TODO: traceability?
WEHMD_API_SOCKET = "/var/run/wehmd_api"


class Wehmd:
    def __init__(self):
        self.socket = socket.socket(family=socket.AF_UNIX,
                                    type=socket.SOCK_STREAM)
        self.socket.connect(WEHMD_API_SOCKET)

    def read_boot_mode(self):
        message = wehmd_msg_t()
        message.command = int(CommandType.WEHMD_CMD_GET_EE_BOOT_MODE)
        message.data.bootMode = 0

        self.send(message)
        response = self.receive()
        return wehmd_msg_t.from_buffer_copy(response)

    def write_boot_mode(self, value):
        message = wehmd_msg_t()
        message.command = int(CommandType.WEHMD_CMD_SET_EE_BOOT_MODE)
        message.data.bootMode = value

        self.send(message)
        response = self.receive()
        return wehmd_msg_t.from_buffer_copy(response)

    def send(self, message):
        raw = (c_char * sizeof(message))()
        memmove(raw, byref(message), sizeof(message))

        total_sent = 0
        while total_sent < len(raw):
            sent = self.socket.send(raw[total_sent:])
            if sent == 0:
                raise RuntimeError("socket connection broken")
            total_sent = total_sent + sent

    def receive(self):
        chunks = []
        bytes_received = 0
        while bytes_received < sizeof(wehmd_msg_t):
            chunk = self.socket.recv(min(sizeof(wehmd_msg_t) - bytes_received, 2048))
            if chunk == b'':
                raise RuntimeError("socket connection broken")
            chunks.append(chunk)
            bytes_received = bytes_received + len(chunk)
        return b''.join(chunks)


CommandType = enum.IntEnum('CommandType', '''
    WEHMD_CMD_GET_OTC
    WEHMD_CMD_OTC
    WEHMD_CMD_GET_EE_PRODUCT_TYPE
    WEHMD_CMD_EE_PRODUCT_TYPE
    WEHMD_CMD_GET_EE_PRODUCT_VARIANT
    WEHMD_CMD_EE_PRODUCT_VARIANT
    WEHMD_CMD_GET_EE_BOARD_REVISION
    WEHMD_CMD_EE_BOARD_REVISION
    WEHMD_CMD_GET_EE_BOARD_NAME
    WEHMD_CMD_EE_BOARD_NAME
    WEHMD_CMD_GET_EE_PART_NUMBER
    WEHMD_CMD_EE_PART_NUMBER
    WEHMD_CMD_GET_EE_SERIAL_NUMBER
    WEHMD_CMD_EE_SERIAL_NUMBER
    WEHMD_CMD_GET_EE_BOOT_MODE
    WEHMD_CMD_EE_BOOT_MODE
    WEHMD_CMD_GET_EE_UNIQUE_RELEASE_ID
    WEHMD_CMD_EE_UNIQUE_RELEASE_ID
    WEHMD_CMD_GET_EE_DELETE_DIAG
    WEHMD_CMD_EE_DELETE_DIAG
    WEHMD_CMD_GET_EE_CONSOLE_MODE
    WEHMD_CMD_EE_CONSOLE_MODE
    WEHMD_CMD_GET_API_VERSION
    WEHMD_CMD_API_VERSION
    WEHMD_CMD_ERROR
    WEHMD_CMD_SET_EE_CONSOLE_MODE
    WEHMD_CMD_GET_EE_DISPLAY_BRIGHTNESS
    WEHMD_CMD_SET_EE_DISPLAY_BRIGHTNESS
    WEHMD_CMD_EE_DISPLAY_BRIGHTNESS
    WEHMD_CMD_GET_EE_KEYPAD_BRIGHTNESS
    WEHMD_CMD_SET_EE_KEYPAD_BRIGHTNESS
    WEHMD_CMD_EE_KEYPAD_BRIGHTNESS
    WEHMD_CMD_SET_EE_BOOT_MODE
    WEHMD_CMD_SET_EE_RAM_TEST
    WEHMD_CMD_GET_EE_RAM_TEST
    WEHMD_CMD_EE_RAM_TEST
    WEHMD_CMD_GET_EE_KEYPAD_COLOR
    WEHMD_CMD_SET_EE_KEYPAD_COLOR
    WEHMD_CMD_EE_KEYPAD_COLOR
''',
    start=0
)


wehmd_cmd_t = c_uint
wehmd_err_type_t = c_uint


class wehmd_error_t(Structure):
    _fields_ = [
        ('command', wehmd_cmd_t),
        ('errorType', wehmd_err_type_t)
    ]


class wehmd_msg_data_t(Union):
    _fields_ = [
        ('otc', c_uint),
        ('productType', c_ubyte),
        ('productVariant', c_ubyte),
        ('boardRevision', c_ubyte),
        ('boardName', c_char * 31),
        ('partNumber', c_char * 21),
        ('serialNumber', c_char * 21),
        ('bootMode', c_ubyte),
        ('uniqueReleaseID', c_uint),
        ('deleteDiag', c_ubyte),
        ('consoleMode', c_ubyte),
        ('apiVersion', c_uint),
        ('displayBrightness', c_ubyte),
        ('keypadBrightness', c_ubyte),
        ('keypadColor', c_uint),
        ('ramTest', c_ubyte),
        ('error', wehmd_error_t)
    ]


class wehmd_msg_t(Structure):
    _fields_ = [
        ('command', wehmd_cmd_t),
        ('data', wehmd_msg_data_t)
    ]


if __name__ == '__main__':
    import sys

    print('No script functionality here')
    sys.exit(1)     # non-zero is a failure



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'listselect.ui'
#
# Created by: PyQt5 UI code generator 5.13.0
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(801, 462)
        self.gridLayout = QtWidgets.QGridLayout(Form)
        self.gridLayout.setContentsMargins(0, 0, 0, 0)
        self.gridLayout.setObjectName("gridLayout")
        self.menu_view = ListMenuView(Form)
        self.menu_view.setObjectName("menu_view")
        self.gridLayout.addWidget(self.menu_view, 0, 0, 3, 1)
        self.accept_button = QtWidgets.QPushButton(Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.accept_button.sizePolicy().hasHeightForWidth())
        self.accept_button.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setFamily("FontAwesome")
        self.accept_button.setFont(font)
        self.accept_button.setObjectName("accept_button")
        self.gridLayout.addWidget(self.accept_button, 0, 2, 1, 1)
        self.cancel_button = QtWidgets.QPushButton(Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.cancel_button.sizePolicy().hasHeightForWidth())
        self.cancel_button.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setFamily("FontAwesome")
        self.cancel_button.setFont(font)
        self.cancel_button.setObjectName("cancel_button")
        self.gridLayout.addWidget(self.cancel_button, 2, 2, 1, 1)
        self.gridLayout.setColumnStretch(0, 1)

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Form"))
        self.accept_button.setText(_translate("Form", ""))
        self.cancel_button.setText(_translate("Form", ""))
from epyqlib.listmenuview import ListMenuView



import contextlib
import functools
import locale
import os
import pathlib
import sys

import attr


library_path = pathlib.Path(__file__).parents[2].resolve()

project_path = library_path.parents[1]

scripts_path = project_path/'venv'/'Scripts'

examples_path = library_path/'examples'/'develop'

device_path = examples_path/'devices'

default_parameters_path = examples_path/'defaults.pmvs'
small_parameters_path = examples_path/'small.pmvs'

devices = {
    'customer': pathlib.Path('customer')/'distributed_generation.epc',
    'factory': pathlib.Path('factory')/'distributed_generation_factory.epc',
}

devices = {
    k: device_path / v
    for k, v in devices.items()
}

symbol_files = {
    'customer': pathlib.Path('customer')/'EPC_DG_ID247.sym',
    'factory': pathlib.Path('factory')/'EPC_DG_ID247_FACTORY.sym',
}

symbol_files = {
    k: device_path / v
    for k, v in symbol_files.items()
}

hierarchy_files = {
    'customer': pathlib.Path('customer')/'EPC_DG_ID247.parameters.json',
    'factory': pathlib.Path('factory')/'EPC_DG_ID247_FACTORY.parameters.json',
}

hierarchy_files = {
    k: device_path / v
    for k, v in hierarchy_files.items()
}


def single(x):
    y, = x

    return y


@attr.s
class DeviceFiles:
    can = attr.ib()
    hierarchy = attr.ib()
    device = attr.ib()
    epp = attr.ib()
    pmvs = attr.ib()

    @classmethod
    def build(cls, base, version, level):
        version = pathlib.Path(base) / version
        path = version / 'devices' / level

        print(path, list(path.glob('*')))

        return cls(
            can=single(path.glob('*.sym')),
            hierarchy=single(path.glob('*.json')),
            device=single(path.glob('*.epc')),
            epp=version/'small.epp',
            pmvs=version/'small.pmvs',
        )


new_examples_path = library_path/'examples'


@functools.lru_cache()
def new_devices():
    return {
        (version, level): DeviceFiles.build(
            base=new_examples_path,
            version=version,
            level=level,
        )
        for version in (
            'develop',
            'v1.2.5',
        )
        for level in (
            'customer',
            'factory',
        )
        if (version, level) not in (
            ('v1.2.5', 'customer'),
        )
    }


@attr.s
class Values:
    initial = attr.ib()
    input = attr.ib()
    expected = attr.ib()
    collected = attr.ib(default=attr.Factory(list))

    def collect(self, value):
        self.collected.append(value)

    def check(self):
        return all(x == y for x, y in zip(self.expected, self.collected))


@contextlib.contextmanager
def use_locale(*s):
    if len(s) == 0:
        s = ('',)

    #use setlocale() to get present settings because reasons:
    #https://docs.python.org/3/library/locale.html#locale.getlocale
    #https://docs.python.org/3/library/locale.html#locale.setlocale
    old = locale.setlocale(locale.LC_ALL)

    for name in s:
        try:
            locale.setlocale(locale.LC_ALL, name)
        except locale.Error:
            continue

        break
    else:
        assert False, 'Unable to set locale to any of {}'.format(s)

    yield

    locale.setlocale(locale.LC_ALL, old)



#!/usr/bin/env python3

#TODO: """DocString if there is one"""

from PyQt5.QtCore import pyqtProperty

# See file COPYING in this source tree
__copyright__ = 'Copyright 2016, EPC Power Corp.'
__license__ = 'GPLv2+'


class OverrideRange:
    def __init__(self, *args, **kwargs):
        self._override_range = False
        self._min = 0
        self._max = 1

        super().__init__(*args, **kwargs)

    @pyqtProperty(bool)
    def override_range(self):
        return self._override_range

    @override_range.setter
    def override_range(self, override):
        self._override_range = bool(override)

    @pyqtProperty(float)
    def minimum(self):
        return self._min

    @minimum.setter
    def minimum(self, min):
        self._min = min

    @pyqtProperty(float)
    def maximum(self):
        return self._max

    @maximum.setter
    def maximum(self, max):
        self._max = max


if __name__ == '__main__':
    import sys

    print('No script functionality here')
    sys.exit(1)     # non-zero is a failure



from PyQt5 import QtDesigner
import epyqlib.listmenuview
import epyqlib.abstractpluginclass

# See file COPYING in this source tree
__copyright__ = 'Copyright 2016, EPC Power Corp.'
__license__ = 'GPLv2+'


class ListMenuViewPlugin(epyqlib.abstractpluginclass.AbstractPlugin):
    def __init__(self, parent=None):
        epyqlib.abstractpluginclass.AbstractPlugin.__init__(self, parent=parent)

        self._group = 'EPC - General'
        self._init = epyqlib.listmenuview.ListMenuView
        self._module_path = 'epyqlib.listmenuview'
        self._name = 'ListMenuView'



# This file has been generated


build_system = 'AppVeyor'
build_id = '29634022'
build_number = '415'
build_version = '1.0.415'
job_id = 'x4mxxasn97s4w95f'
job_url = 'https://ci.appveyor.com/project/KyleAltendorf/stlib/build/job/x4mxxasn97s4w95f'



#!/usr/bin/env python3

#TODO: """DocString if there is one"""

import collections
import functools
import io
import pathlib

from PyQt5 import QtWidgets, QtCore
from PyQt5.QtCore import pyqtSignal, pyqtSlot, Qt, QAbstractProxyModel
import twisted.internet.defer

import epyqlib.autodevice.build
import epyqlib.nv
import epyqlib.nvview_ui
try:
    import epyqlib.resources.code
except ImportError:
    pass # we will catch the failure to open the file
import epyqlib.utils.qt


# See file COPYING in this source tree
__copyright__ = 'Copyright 2018, EPC Power Corp.'
__license__ = 'GPLv2+'


class ActivityError(Exception):
    pass


class NvView(QtWidgets.QWidget):
    module_to_nv = pyqtSignal()
    read_from_file = pyqtSignal()
    read_from_value_set_file = pyqtSignal()
    write_to_file = pyqtSignal()
    write_to_value_set_file = pyqtSignal()
    auto_read_checked = pyqtSignal()
    auto_read_unchecked = pyqtSignal()

    def __init__(self, parent=None, in_designer=False):
        super().__init__(parent=parent)

        self.in_designer = in_designer

        self.ui = epyqlib.nvview_ui.Ui_Form()
        self.ui.setupUi(self)

        self.ui.module_to_nv_button.clicked.connect(self.module_to_nv)
        self.ui.write_to_module_button.clicked.connect(self.write_to_module)
        self.ui.read_from_module_button.clicked.connect(self.read_from_module)
        self.ui.write_to_file_button.clicked.connect(self.write_to_file)
        self.ui.write_to_value_set_file_button.clicked.connect(
            self.write_to_value_set_file,
        )
        self.ui.read_from_file_button.clicked.connect(self.read_from_file)
        self.ui.read_from_value_set_file_button.clicked.connect(
            self.read_from_value_set_file,
        )
        self.ui.write_to_auto_parameters_button.clicked.connect(
            self.write_to_auto_parameters,
        )

        view = self.ui.tree_view

        view.setContextMenuPolicy(Qt.CustomContextMenu)
        view.customContextMenuRequested.connect(self.context_menu)

        view.header().setContextMenuPolicy(Qt.CustomContextMenu)
        view.header().customContextMenuRequested.connect(
            self.header_context_menu,
        )

        view.setSelectionBehavior(view.SelectItems)
        view.setSelectionMode(view.ExtendedSelection)
        view.row_columns = {
            epyqlib.nv.Columns.indexes.name,
        }
        self.meta_columns = {
            getattr(epyqlib.nv.Columns.indexes, meta.name)
            for meta in epyqlib.nv.MetaEnum
        }
        no_update_columns = set(epyqlib.nv.Columns.indexes)
        no_update_columns -= {epyqlib.nv.Columns.indexes.name,}
        no_update_columns -= self.meta_columns
        no_update_columns.remove(epyqlib.nv.Columns.indexes.scratch)
        view.no_update_columns = no_update_columns

        self.resize_columns = epyqlib.nv.Columns(
            name=True,
            comment=True,
        )

        self.progress = None

        self.ui.tree_view.clicked.connect(self.clicked)
        self.ui.tree_view.header().setMinimumSectionSize(0)

        self.ui.searchbox.connect_to_view(
            view=self.ui.tree_view,
            column=epyqlib.nv.Columns.indexes.name,
        )

        self.can_contents = None
        self.can_suffix = None
        self.set_access_level_signal_path(path=None)

        self.password_mapper = QtWidgets.QDataWidgetMapper()
        self.password_mapper.setSubmitPolicy(
            QtWidgets.QDataWidgetMapper.ManualSubmit,
        )
        self.access_level_mapper = QtWidgets.QDataWidgetMapper()
        self.access_level_mapper.setSubmitPolicy(
            QtWidgets.QDataWidgetMapper.ManualSubmit,
        )

        self.ui.access_level_password.setPlaceholderText('Access Code...')

        self.metas = None
        self.set_metas(())

        self.resize_modes = None

        self.device = None

        self.diff_proxy = None

        self.ui.diff_reference_column.currentIndexChanged[int].connect(
            self.diff_reference_column_changed,
        )

        self.ui.auto_read.stateChanged.connect(
            lambda state: (
                self.auto_read_checked.emit()
                if state == Qt.Checked
                else self.auto_read_unchecked.emit()
            )
        )

    def terminate(self):
        self.device = None

    def configure_diff_proxy(self, proxy):
        self.diff_proxy = proxy
        self.diff_reference_column_changed(
            self.ui.diff_reference_column.currentIndex(),
        )

    def diff_reference_column_changed(self, index):
        if self.diff_proxy is not None:
            column = self.ui.diff_reference_column.itemData(
                index,
                epyqlib.utils.qt.UserRoles.raw,
            )
            self.diff_proxy.reference_column = column

    def set_device(self, device):
        self.device = device

    def set_access_level_signal_path(self, path):
        if path is None or path == '':
            self.ui.current_access_level.signal_path = ''
            self.ui.current_access_level.setHidden(True)
            self.ui.access_level.setHidden(True)
            self.ui.current_access_level.ignore = True
        else:
            self.ui.current_access_level.signal_path = ';'.join(path)
            self.ui.current_access_level.setVisible(True)
            self.ui.access_level.setVisible(True)
            self.ui.current_access_level.ignore = False

    def set_metas(self, metas):
        self.metas = metas

        show_but_no_edit = {
            epyqlib.nv.MetaEnum.factory_default,
            epyqlib.nv.MetaEnum.minimum,
            epyqlib.nv.MetaEnum.maximum,
        }

        model = self.nonproxy_model()
        if model is not None:
            for meta in show_but_no_edit:
                index = epyqlib.nv.column_index_by_meta[meta]
                model.editable_columns[index] = (
                    meta in self.metas
                )

        for meta in set(epyqlib.nv.MetaEnum) - show_but_no_edit:
            self.ui.tree_view.setColumnHidden(
                epyqlib.nv.column_index_by_meta[meta],
                meta not in self.metas,
            )

        if set(self.metas) == {epyqlib.nv.MetaEnum.value}:
            self.ui.tree_view.row_columns = set(epyqlib.nv.Columns.indexes)
        else:
            self.ui.tree_view.row_columns = {
                epyqlib.nv.Columns.indexes.name,
            }

        self.update_diff_reference_columns()

    def filter_text_changed(self, text):
        self.ui.tree_view.model().setFilterWildcard(text)

    # TODO: CAMPid 07943342700734207878034207087
    def nonproxy_model(self):
        model = self.ui.tree_view.model()
        while isinstance(model, QAbstractProxyModel):
            model = model.sourceModel()

        return model

    def set_sorting_enabled(self, enabled):
        self.ui.tree_view.setSortingEnabled(enabled)

    def sort_by_column(self, column, order):
        self.ui.tree_view.sortByColumn(column, order)

    def write_to_module(self):
        model = self.nonproxy_model()

        def not_none(nv):
            if nv.value is not None:
                return True

            return any(
                getattr(nv.meta, meta.name).value is not None
                for meta in epyqlib.nv.MetaEnum.non_value
            )

        only_these = [nv for nv in model.all_nv()
                      if not_none(nv) is not None]
        callback = functools.partial(
            self.update_signals,
            only_these=only_these
        )
        d = model.root.write_all_to_device(
            callback=callback,
            only_these=only_these,
            meta=tuple(
                meta
                for meta in epyqlib.nv.meta_limits_first
                if meta in self.metas
            ),
        )
        d.addErrback(epyqlib.utils.twisted.catch_expected)
        d.addErrback(epyqlib.utils.twisted.errbackhook)

    def disable_column_resize(self):
        self.resize_modes = {
            index: (
                self.ui.tree_view.header().sectionResizeMode(
                    index,
                )
            )
            for index in epyqlib.nv.column_index_by_meta.values()
        }

        for index in self.resize_modes:
            self.ui.tree_view.header().setSectionResizeMode(
                index,
                QtWidgets.QHeaderView.Fixed,
            )

    def enable_column_resize(self):
        for index, mode in self.resize_modes.items():
            self.ui.tree_view.header().setSectionResizeMode(
                index,
                mode,
            )

    def read_from_module(self):
        self.disable_column_resize()

        model = self.nonproxy_model()
        only_these = [nv for nv in model.all_nv()]
        callback = functools.partial(
            self.update_signals,
            only_these=only_these
        )
        d = model.root.read_all_from_device(
            callback=callback,
            only_these=only_these,
            meta=tuple(
                meta
                for meta in epyqlib.nv.meta_limits_first
                if meta in self.metas
            ),
        )

        d.addBoth(
            epyqlib.utils.twisted.detour_result,
            self.enable_column_resize,
        )
        d.addErrback(epyqlib.utils.twisted.catch_expected)
        d.addErrback(epyqlib.utils.twisted.errbackhook)

    def write_to_auto_parameters(self):
        builder = epyqlib.autodevice.build.Builder()

        root = self.nonproxy_model().root

        def name_from_node(node):
            if node is None:
                return None

            return ':'.join((node.frame.mux_name, node.name))

        builder.set_access_level_names(
            password_name=name_from_node(root.password_node),
            access_level_name=name_from_node(root.access_level_node),
        )

        auto_parameters_device_file_path = epyqlib.utils.qt.file_dialog(
            filters=[
                ('EPC Auto Device Template Archive', ['zip']),
                ('EPC Auto Device Template', ['epc']),
                ('All Files', ['*'])
            ],
            caption='Open Auto Parameters Template',
            parent=self,
            path_factory=pathlib.Path,
        )
        if auto_parameters_device_file_path is None:
            return

        builder.set_template(
            path=auto_parameters_device_file_path,
            archive=(
                auto_parameters_device_file_path.suffix.casefold() == '.zip'
            ),
        )

        parameter_source_path = epyqlib.utils.qt.file_dialog(
            filters=[
                ('Parameter Value Set', ['pmvs']),
                ('EPC Parameters', ['epp']),
            ],
            caption='Open Parameter Or Value Set File',
            parent=self,
            path_factory=pathlib.Path,
        )
        if parameter_source_path is None:
            return

        builder.set_original_raw_dict(self.device.raw_dict)

        if parameter_source_path.suffix == '.pmvs':
            builder.load_pmvs(parameter_source_path)
        elif parameter_source_path.suffix == '.epp':
            with open(parameter_source_path) as parameters:
                builder.load_epp(
                    can=io.BytesIO(self.can_contents),
                    can_suffix=self.can_suffix,
                    parameters=parameters,
                )
        else:
            raise Exception("Must pick either a *.pmvs or *.epp file")

        filters = [
            ('EPC Device', ['epz']),
            ('All Files', ['*'])
        ]
        filename = epyqlib.utils.qt.file_dialog(
            filters,
            save=True,
            parent=self,
        )

        if filename is None:
            return

        builder.set_target(path=filename)

        if builder.archive_code is None:
            archive_code, ok = QtWidgets.QInputDialog.getText(
                None,
                '.epz Password',
                '.epz Password (empty for no password)',
                QtWidgets.QLineEdit.Password,
            )

            if not ok:
                return

            builder.archive_code = archive_code

        for access_input in builder.access_parameters:
            if access_input.node is None:
                continue

            parameters = [
                None,
                access_input.description,
                access_input.description,
            ]

            if access_input.secret:
                parameters.append(QtWidgets.QLineEdit.Password)

            user_input, ok = QtWidgets.QInputDialog.getText(*parameters)

            if not ok:
                return

            access_input.value = int(user_input)

        text = ''

        while True:
            dialog = QtWidgets.QInputDialog(self)
            dialog.setInputMode(QtWidgets.QInputDialog.TextInput)
            dialog.setOkButtonText('Yes')
            dialog.setCancelButtonText('No')
            dialog.setWindowTitle('Serial Number Lock')
            dialog.setLabelText('Lock to a specific serial number?')
            dialog.setTextValue(text)

            if not dialog.exec():
                break

            text = dialog.textValue()

            try:
                builder.required_serial_number = int(text)
            except ValueError:
                epyqlib.utils.qt.dialog(
                    parent=self,
                    message=(
                        f'Unable to parse input {text!r}, must be an integer'
                    ),
                )
            else:
                break

        builder.create(can_contents=self.can_contents)

    def set_can_contents(self, can_contents, suffix):
        self.can_contents = can_contents
        self.can_suffix = suffix

    def configure_sort_proxy(self, proxy):
        proxy.setSortRole(epyqlib.utils.qt.UserRoles.sort)

    def setModel(self, model):
        self.ui.tree_view.setModel(model)
        model = self.nonproxy_model()

        if model.root.password_node is not None:
            self.password_mapper.setModel(model)
            self.password_mapper.setRootIndex(model.index_from_node(
                model.root.password_node.tree_parent,
            ))
            self.password_mapper.setCurrentIndex(
                model.index_from_node(model.root.password_node).row(),
            )
            self.password_mapper.addMapping(
                self.ui.access_level_password,
                epyqlib.nv.Columns.indexes.value,
            )

        access_level_node = model.root.access_level_node
        if access_level_node is not None:
            self.access_level_mapper.setModel(model)
            self.access_level_mapper.setRootIndex(model.index_from_node(
                access_level_node.tree_parent,
            ))
            access_level_index = model.index_from_node(access_level_node)
            self.access_level_mapper.setCurrentIndex(
                access_level_index.row(),
            )

            # TODO: CAMPid 9754542524161542698615426
            # TODO: use the userdata to make it easier to get in and out
            self.ui.access_level.addItems(
                model.root.access_level_node.enumeration_strings(
                    include_values=True,
                ),
            )

            self.access_level_mapper.addMapping(
                self.ui.access_level,
                epyqlib.nv.Columns.indexes.value,
                'currentIndex'.encode('utf-8'),
            )

            delegate = self.access_level_mapper.itemDelegate()
            self.ui.access_level.currentIndexChanged.connect(
                lambda: delegate.commitData.emit(self.ui.access_level),
            )

        selected_nodes = tuple(
            node
            for node in (
                model.root.password_node,
                model.root.access_level_node,
            )
            if node is not None
        )

        def write_access_level():
            model.start_transaction()

            self.password_mapper.submit()
            self.access_level_mapper.submit()

            model.submit_transaction()

        self.ui.set_access_level.clicked.connect(write_access_level)

        model.activity_started.connect(self.activity_started)
        model.activity_ended.connect(self.activity_ended)

        self.ui.enforce_range_limits_check_box.stateChanged.connect(
            model.check_range_changed,
        )
        model.check_range_changed(
            self.ui.enforce_range_limits_check_box.checkState(),
        )

        self.module_to_nv.connect(model.module_to_nv)

        read_from_file = functools.partial(
            model.read_from_file,
            parent=self
        )
        self.read_from_file.connect(read_from_file)

        read_from_value_set_file = functools.partial(
            model.read_from_value_set_file,
            parent=self
        )
        self.read_from_value_set_file.connect(read_from_value_set_file)

        write_to_file = functools.partial(
            model.write_to_file,
            parent=self
        )
        self.write_to_file.connect(write_to_file)

        write_to_value_set_file = functools.partial(
            model.write_to_value_set_file,
            parent=self
        )
        self.write_to_value_set_file.connect(write_to_value_set_file)

        for i in epyqlib.nv.Columns.indexes:
            if self.resize_columns[i]:
                self.ui.tree_view.header().setSectionResizeMode(
                    i, QtWidgets.QHeaderView.ResizeToContents)

        for column in (*model.meta_columns, epyqlib.nv.Columns.indexes.scratch):
            self.ui.tree_view.setItemDelegateForColumn(
                column,
                epyqlib.delegates.ByFunction(
                    model=model,
                    parent=self,
                )
            )

        self.ui.tree_view.setColumnHidden(
            epyqlib.nv.Columns.indexes.factory,
            not any(nv.is_factory() for nv in model.root.all_nv())
        )

        model.force_action_decorations = True
        for column in model.icon_columns:
            self.ui.tree_view.resizeColumnToContents(column)
            self.ui.tree_view.header().setSectionResizeMode(
                column, QtWidgets.QHeaderView.Fixed)

        max_icon_column_width = max(
            self.ui.tree_view.columnWidth(c) for c in model.icon_columns
        )

        for column in model.icon_columns:
            self.ui.tree_view.header().setMinimumSectionSize(0)
            self.ui.tree_view.setColumnWidth(column, max_icon_column_width)

        model.force_action_decorations = False

        self.update_diff_reference_columns()

        self.auto_read_unchecked.connect(
            lambda: model.root.cyclic_reader.pause(self),
        )
        self.auto_read_checked.connect(
            lambda: model.root.cyclic_reader.unpause(self),
        )
        if self.ui.auto_read.checkState() == Qt.Checked:
            model.root.cyclic_reader.unpause(self)
        else:
            model.root.cyclic_reader.pause(self)


    def update_diff_reference_columns(self):
        model = self.nonproxy_model()

        self.ui.diff_reference_column.clear()

        if model is None:
            return

        items = (
            ('No Diff', None),
            *(
                (model.headers[column], column)
                for column in epyqlib.nv.diffable_columns
                if not self.ui.tree_view.isColumnHidden(column)
            )
        )

        default_reference_preference = [
            epyqlib.nv.Columns.indexes.user_default,
            epyqlib.nv.Columns.indexes.factory_default,
            epyqlib.nv.Columns.indexes.scratch,
        ]

        default_reference = next(
            index
            for index in default_reference_preference
            if not self.ui.tree_view.isColumnHidden(index)
        )

        for i, (text, value) in enumerate(items):
            self.ui.diff_reference_column.addItem(text)
            self.ui.diff_reference_column.setItemData(
                i,
                value,
                epyqlib.utils.qt.UserRoles.raw,
            )
            if value == default_reference:
                self.ui.diff_reference_column.setCurrentIndex(i)

    def clicked(self, index):
        index = epyqlib.utils.qt.resolve_index_to_model(index)
        model = index.model()
        node = model.node_from_index(index)

        if isinstance(node, epyqlib.nv.Nv):
            column = index.column()
            if column == model.headers.indexes.saturate:
                model.saturate_node(node)
            elif column == model.headers.indexes.reset:
                model.reset_node(node)
            elif column == model.headers.indexes.clear:
                model.clear_node(node)

    @pyqtSlot(str)
    def activity_started(self, string):
        self.ui.status_label.setText(string)
        if self.progress is not None:
            raise ActivityError(
                'New activity started while another is active: '
                f'new: {string}'
            )
        self.progress = epyqlib.utils.qt.Progress()
        self.progress.connect(
            progress=epyqlib.utils.qt.progress_dialog(parent=self),
            label_text=string,
        )

    @pyqtSlot(str)
    def activity_ended(self, string):
        self.ui.status_label.setText(string)
        if self.progress is not None:
            self.progress.complete()
            self.progress = None

    def context_menu(self, position):
        index = self.ui.tree_view.indexAt(position)
        if index.isValid():
            index = epyqlib.utils.qt.resolve_index_to_model(index)
            model = index.model()

            node = model.node_from_index(index)
            node_type = type(node)
        else:
            node_type = None

        dispatch = {
            epyqlib.nv.Nv: self.nv_context_menu
        }

        f = dispatch.get(node_type)
        if f is not None:
            f(position)
        else:
            self.other_context_menu(position)

    def other_context_menu(self, position):
        menu = QtWidgets.QMenu(parent=self.ui.tree_view)
        menu.setSeparatorsCollapsible(True)

        expand_all = menu.addAction('Expand All')
        collapse_all = menu.addAction('Collapse All')

        action = menu.exec(self.ui.tree_view.viewport().mapToGlobal(position))

        if action is expand_all:
            self.ui.tree_view.expandAll()
        elif action is collapse_all:
            self.ui.tree_view.collapseAll()

    def nv_context_menu(self, position):
        index = self.ui.tree_view.indexAt(position)
        index = epyqlib.utils.qt.resolve_index_to_model(index)
        model = index.model()

        selection_model = self.ui.tree_view.selectionModel()
        selected_indexes = selection_model.selectedIndexes()
        selected_indexes = tuple(
            epyqlib.utils.qt.resolve_index_to_model(i)
            for i in selected_indexes
        )

        selected_by_node = collections.defaultdict(list)
        selected_by_meta = collections.defaultdict(list)
        selected_column_by_node = collections.defaultdict(set)

        for index in selected_indexes:
            node = model.node_from_index(index)
            column = index.column()

            selected_column_by_node[node].add(column)

            if not isinstance(node, epyqlib.nv.Nv):
                continue
            if column not in self.meta_columns:
                continue

            meta = getattr(
                epyqlib.nv.MetaEnum,
                epyqlib.nv.Columns().index_from_attribute(index.column()),
            )
            if meta not in self.metas:
                meta = epyqlib.nv.MetaEnum.value

            if meta not in selected_by_node[node]:
                selected_by_node[node].append(meta)

            if node not in selected_by_meta[meta]:
                selected_by_meta[meta].append(node)

        menu = QtWidgets.QMenu(parent=self.ui.tree_view)
        menu.setSeparatorsCollapsible(True)

        read = menu.addAction('Read {}'.format(
            self.ui.read_from_module_button.text()))
        write = menu.addAction('Write {}'.format(
            self.ui.write_to_module_button.text()))
        saturate = menu.addAction('Saturate')

        def can_be(method_name, selected):
            return any(
                any(
                    getattr(n, method_name)(meta=meta)
                    for meta in metas
                )
                for n, metas in selected.items()
            )

        if not can_be('can_be_saturated', selected_by_node):
            saturate.setDisabled(True)
        reset = menu.addAction('Reset')
        if not can_be('can_be_reset', selected_by_node):
            reset.setDisabled(True)
        clear = menu.addAction('Clear')
        if not can_be('can_be_cleared', selected_by_node):
            clear.setDisabled(True)

        copy_to_column_menu = menu.addMenu('Copy To Column')
        copy_to_column_menu.setDisabled(
            any(len(metas) > 1 for metas in selected_column_by_node.values()),
        )
        copy_to_columns = {
            copy_to_column_menu.addAction(model.headers[column]): column
            for column in (
                *epyqlib.nv.meta_column_indexes,
                epyqlib.nv.Columns.indexes.scratch,
            )
        }

        menu.addSeparator()
        expand_all = menu.addAction('Expand All')
        collapse_all = menu.addAction('Collapse All')

        action = menu.exec(self.ui.tree_view.viewport().mapToGlobal(position))

        d = twisted.internet.defer.Deferred()
        d.callback(None)

        if action in copy_to_columns:
            destination_column = copy_to_columns[action]
            model.start_transaction()

            for index in selected_indexes:
                maybe_copy(
                    source_column=index.column(),
                    destination_column=destination_column,
                    index=index,
                    model=model,
                )

            model.submit_transaction()
        else:
            for meta, nodes in selected_by_meta.items():
                callback = functools.partial(
                    self.update_signals,
                    only_these=nodes,
                )
                if action is None:
                    pass
                elif action is read:
                    d.addCallback(
                        lambda _, nodes=nodes, callback=callback, meta=meta:
                        model.root.read_all_from_device(
                            only_these=nodes,
                            callback=callback,
                            meta=(meta,),
                        )
                    )
                elif action is write:
                    d.addCallback(
                        lambda _, nodes=nodes, callback=callback, meta=meta:
                        model.root.write_all_to_device(
                            only_these=nodes,
                            callback=callback,
                            meta=(meta,),
                        )
                    )
                elif action is saturate:
                    self.disable_column_resize()
                    for node in nodes:
                        model.saturate_node(node, meta=meta)
                    self.enable_column_resize()
                elif action is reset:
                    self.disable_column_resize()
                    for node in nodes:
                        model.reset_node(node, meta=meta)
                    self.enable_column_resize()
                elif action is clear:
                    self.disable_column_resize()
                    for node in nodes:
                        model.clear_node(node, meta=meta)
                    self.enable_column_resize()
                elif action is expand_all:
                    self.ui.tree_view.expandAll()
                elif action is collapse_all:
                    self.ui.tree_view.collapseAll()

        d.addErrback(epyqlib.utils.twisted.catch_expected)
        d.addErrback(epyqlib.utils.twisted.errbackhook)

    def header_context_menu(self, pos):
        menu = QtWidgets.QMenu(parent=self.ui.tree_view)

        source_column = self.ui.tree_view.header().logicalIndexAt(pos)

        model = self.nonproxy_model()

        copy_to_column_menu = menu.addMenu('Copy To Column')
        copy_to_columns = {
            copy_to_column_menu.addAction(
                model.headers[column],
            ): column
            for column in (
                *epyqlib.nv.meta_column_indexes,
                epyqlib.nv.Columns.indexes.scratch,
            )
        }
        this_action, = (
            k
            for k, v in copy_to_columns.items()
            if v == source_column
        )
        this_action.setDisabled(True)

        action = menu.exec(self.ui.tree_view.mapToGlobal(pos))

        if action in copy_to_columns:
            destination_column = copy_to_columns[action]
            model.start_transaction()
            for node in model.all_nv():
                index = model.index_from_node(node)
                maybe_copy(
                    source_column=source_column,
                    destination_column=destination_column,
                    index=index,
                    model=model,
                )

            model.submit_transaction()

    # TODO: CAMPid 0347987975t427567139419439349
    def update_signals(self, arg, only_these):
        d, meta = arg
        model = self.nonproxy_model()

        frame = next(iter(d)).frame

        signals = set(only_these)
        signals &= set(frame.set_frame.parameter_signals)

        for signal in signals:
            if signal.status_signal in d:
                if not signal.status_signal.write_only:
                    value = d[signal.status_signal]
                    signal.set_meta(value, meta=meta, check_range=False)
                    signal.set_from_device(
                        column=getattr(epyqlib.nv.Columns.indexes, meta.name),
                    )

        for signal in frame.set_frame.parameter_signals:
            QtWidgets.QApplication.instance().processEvents()

            model.dynamic_columns_changed(
                signal,
                columns=(getattr(epyqlib.nv.Columns.indexes, meta.name),)
            )


def maybe_copy(source_column, destination_column, index, model):
    data = index.siblingAtColumn(source_column).data(role=Qt.EditRole)

    skip = (
        data is None
        or data == ''
        or model.node_from_index(index).secret
    )

    if skip:
        return

    model.setData(
        index=index.siblingAtColumn(destination_column),
        data=data,
        role=Qt.EditRole,
    )


if __name__ == '__main__':
    import sys

    print('No script functionality here')
    sys.exit(1)     # non-zero is a failure



#!/usr/bin/env python3

import epyqlib.twisted.cancalibrationprotocol as ccp
import argparse


class Frame:
    def __init__(self, index, time, type, id, data, comment):
        self.index = int(index)
        self.time = float(time)
        self.type = type
        self.id = (int(id, 16), len(id) > 3)
        self.data = [int(d, 16) for d in data]
        self.comment = comment

    @classmethod
    def len(cls):
        return 6

    def __str__(self):
        format = ('Index: {{}}, Time: {{:.4f}}, Type: {{}}, ID: {{:0{}X}}, '
                  'Data: {{}}, Comment: {{}}').format(8 if self.id[1] else 3)
        data = ['{:02X}'.format(d) for d in self.data]
        data = ' '.join(data)
        return format.format(self.index, self.time, self.type, self.id[0],
                             data, self.comment)


def load_trc(file):
    frame_length = Frame.len()

    for line in file:
        if line.lstrip().startswith(';'):
            continue

        elements = line.replace(')', '').split()
        data_length = int(elements[frame_length - 2])
        meta = elements[:frame_length - 2]
        data = elements[frame_length - 1:frame_length + data_length - 1]
        try:
            comment = elements[frame_length + data_length - 1]
        except IndexError:
            comment = None

        frame = Frame(*meta, data, comment)
        if data_length != len(frame.data):
            raise Exception('blue')

        frame.time /= 1000

        yield frame


def load_sock(file):
    frame_length = Frame.len()

    for line in file:
        elements = line.replace('(', '').replace(')', '').replace('#', ' ').split()
        id = elements[2]
        if id == 'FFFFFFFF':
            comment = elements[3].lstrip('0123456789')
            data = elements[3][:-len(comment)]
        else:
            data = elements[3]
            data = [data[i:i+2] for i in range(0, len(data), 2)]

        frame = Frame(index=0,
                      time=elements[0],
                      type='Rx',
                      id=id,
                      data=data,
                      comment=None)

        yield frame


def parse_args(args):
    parser = argparse.ArgumentParser()
    parser.add_argument('--trace', '-t',
                        type=argparse.FileType('r'))
    parser.add_argument('--sock', '-s',
                        type=argparse.FileType('r'))

    return parser.parse_args(args)


def main(args=None):
    if args is None:
        args = sys.argv[1:]

    args = parse_args(args=args)

    blocks = []

    if not ((args.trace is not None) ^ (args.sock is not None)):
        print('You must specify one and only one file type')
        return -1

    if args.trace:
        frames = load_trc(args.trace)
    elif args.sock:
        frames = load_sock(args.sock)

    for frame in frames:
        if (frame.type == 'Rx'
                and frame.id[1]
                and frame.id[0] == ccp.bootloader_can_id):
            if frame.data[0] == ccp.CommandCode.set_mta:
                address = sum(v << (8*(3-p)) for p, v in enumerate(frame.data[-4:]))
                # print('Raw Address: {}'.format(frame.data[-4:]))
                # print('Address: {}'.format(address))
                # if len(blocks) != 0:
                #     print('Latest block: {}'.format(blocks[-1]))
                #     print('Latest start: {}'.format(blocks[-1][0]))
                #     print('Latest length: {}'.format(blocks[-1][1]))
                if (len(blocks) == 0
                        or blocks[-1][0] + blocks[-1][1]/2 != address):
                    blocks.append([address, 0])
            elif frame.data[0] == ccp.CommandCode.download_6:
                blocks[-1][1] += 6
            elif frame.data[0] == ccp.CommandCode.download:
                blocks[-1][1] += frame.data[2]

    print('Note that separate blocks will be detected as one if they are contiguous.')
    for block in blocks:
        print('Start: {:08X}, Length: {}, Bytes: {}'.format(
            block[0], int(block[1]/2), block[1]
        ))


if __name__ == '__main__':
    import sys

    sys.exit(main())



#!/usr/bin/env python3

#TODO: """DocString if there is one"""

import epyqlib.widgets.abstractwidget
import epyqlib.widgets.progressbar_ui


# See file COPYING in this source tree
__copyright__ = 'Copyright 2016, EPC Power Corp.'
__license__ = 'GPLv2+'


class ProgressBar(epyqlib.widgets.abstractwidget.AbstractWidget):
    def __init__(self, parent=None, in_designer=False):
        self._min = None
        self._max = None
        self._counts = 1000
        self._frame = None
        self._signal = None

        super().__init__(
            ui_class=epyqlib.widgets.progressbar_ui.Ui_Form,
            parent=parent,
            in_designer=in_designer,
        )

        self.ui.progressBar.setRange(0, self._counts)

    def set_value(self, value):
        if value is None:
            value = 0

        if value >= 0:
            # TODO: fix this especially for positive and negative ranges
            counts = value - self._min
        elif value < 0:
            counts = abs(value) - abs(self._max)

        counts /= abs(self._max - self._min)
        counts *= self._counts
        self.ui.progressBar.setValue(int(round(counts)))
        # TODO: quit hardcoding this and it's better implemented elsewhere
        self.ui.progressBar.setFormat('{0:.2f}'.format(value))

    def set_range(self, min=None, max=None):
        if min * max < 0:
            # opposite signs and neither is zero
            # TODO: pick the right exception
            raise Exception('Signs must match or one limit be zero')
        elif min == max:
            # TODO: pick the right exception
            raise Exception('Min and max may not be the same')
        elif min > max:
            # TODO: pick the right exception
            raise Exception('Min must be less than max')

        self._min = min
        self._max = max
        self.ui.progressBar.setInvertedAppearance(self._min < 0)
        # self.ui.progressBar.setRange(self._min, self._max)



if __name__ == '__main__':
    import sys

    print('No script functionality here')
    sys.exit(1)     # non-zero is a failure



#!/usr/bin/env python3

#TODO: """DocString if there is one"""

import canmatrix.formats
import epyqlib.canneo
import epyqlib.widgets.abstractwidget
import math
import os
import sys

from PyQt5.QtCore import pyqtProperty, QTimer
from PyQt5.QtWidgets import QWidget

# See file COPYING in this source tree
__copyright__ = 'Copyright 2016, EPC Power Corp.'
__license__ = 'GPLv2+'


class EpcForm(QWidget):
    def __init__(self, parent=None, in_designer=False):
        self.in_designer = in_designer

        QWidget.__init__(self, parent=parent)

        self._can_file = ''
        self.form_window = None

        self.neo = None

        self.setGeometry(0, 0, 640, 480)

        self.update()
        # Total hack, but it 'works'.  Should be in response to widgets
        # being added, or done being added, or...
        if self.in_designer:
            QTimer.singleShot(1000, self.update)

    @pyqtProperty(str)
    def can_file(self):
        return self._can_file

    @can_file.setter
    def can_file(self, file):
        self._can_file = file

        self.update()

    @pyqtProperty(bool)
    def update_from_can_file(self):
        return False

    @update_from_can_file.setter
    def update_from_can_file(self, _):
        print('updating now')

        self.update()

    def form_window_file_name_changed(self, _):
        self.update()

    def update(self):
        if not self.in_designer:
            return

        from PyQt5.QtDesigner import QDesignerFormWindowInterface

        can_file = self.can_file

        imported = ()
        new_form_window = (
            QDesignerFormWindowInterface.findFormWindow(self))
        if self.neo is None:
            if self.form_window is not None:
                self.form_window.fileNameChanged.disconnect(
                    self.form_window_file_name_changed)

            self.form_window = new_form_window

            if self.form_window is not None:
                self.form_window.fileNameChanged.connect(
                    self.form_window_file_name_changed)

            if self.form_window is not None:
                if not os.path.isabs(can_file):
                    can_file = os.path.join(
                        os.path.dirname(self.form_window.fileName()),
                        can_file
                    )

            try:
                imported = list(canmatrix.formats.loadp(can_file).values())
            except (FileNotFoundError, IsADirectoryError, OSError):
                # Windows raises an OSError for at least my VirtualBox
                # network drive.
                #
                # >>> f = open('W:/t/603/Hydra_06092017 - old\\'); f.close()
                # Traceback (most recent call last):
                #   File "<stdin>", line 1, in <module>
                # OSError: [Errno 22] Invalid argument: 'W:/t/603/Hydra_06092017 - old\\'
                # >>> f = open('C:/t/603/Hydra_06092017 - old\\'); f.close()
                # Traceback (most recent call last):
                #   File "<stdin>", line 1, in <module>
                # FileNotFoundError: [Errno 2] No such file or directory: 'C:/t/603/Hydra_06092017 - old\\'

                pass

        widgets = self.findChildren(
                epyqlib.widgets.abstractwidget.AbstractWidget)

        if len(imported) > 0:
            self.neo = epyqlib.canneo.Neo(matrix=imported[0])

        for widget in widgets:
            self.update_widget(widget=widget)

    def update_widget(self, widget):
        if not self.in_designer:
            return

        if self.neo is None:
            widget.set_signal(force_update=True)
        else:
            # TODO: CAMPid 07340793413419714301373147
            widget.set_range(min=0, max=100)
            widget.set_value(math.nan)

            frame = widget.property('frame')
            if frame is not None:
                signal = widget.property('signal')
                signal_path = (frame, signal)
            else:
                signal_path = tuple(
                    e for e in widget._signal_path if len(e) > 0)

            try:
                signal = self.neo.signal_by_path(*signal_path)
            except epyqlib.canneo.NotFoundError:
                pass
            else:
                widget.set_signal(signal)


if __name__ == '__main__':
    import sys

    print('No script functionality here')
    sys.exit(1)     # non-zero is a failure



import datetime
import pathlib
import subprocess
import sys
import textwrap

import pkg_resources
import click


@click.command()
@click.option('--dist', '--distribution', '-d', 'distribution', required=True)
@click.option('--root', '-r', default=None)
@click.option('--out', '-o', type=click.File('w'), default=sys.stdout)
def cli(distribution, root, out):
    version = pkg_resources.get_distribution(distribution).version

    sha = subprocess.check_output(
        [
            'git',
            'rev-parse',
            '--verify',
            '--quiet',
            'HEAD',
        ],
        cwd=root,
        encoding='utf-8',
    ).strip()

    actual_root_path = subprocess.check_output(
        [
            'git',
            'rev-parse',
            '--show-toplevel',
        ],
        cwd=root,
        encoding='utf-8',
    ).strip()

    file_path = pathlib.Path(__file__).relative_to(actual_root_path)

    contents = textwrap.dedent(f'''\
    # This file was automatically generated by {file_path}
    __version_build_time__ = '{datetime.datetime.utcnow().isoformat()}'
    
    __version__ = '{version}'
    __sha__ = '{sha}'
    __revision__ = '{sha}'
    ''')

    out.write(contents)



import collections
import enum
import logging
import queue
import textwrap
import time

import attr
import twisted.internet.defer
import twisted.protocols.policies

import epyqlib.nv
import epyqlib.utils.general

__copyright__ = 'Copyright 2016, EPC Power Corp.'
__license__ = 'GPLv2+'


logger = logging.getLogger(__name__)


class RequestTimeoutError(epyqlib.utils.general.ExpectedException):
    def __init__(self, state, item):
        message = (
            'Protocol timed out while in state {} handling {}'
            .format(state.name, item)
        )

        super().__init__(message)
        self.state = state
        self.item = item

    def expected_message(self):
        return textwrap.dedent('''\
            Request timed out:
                {}, {}

            1. Confirm converter and adapter are on the same bus
            2. Confirm device file was loaded with node ID matching the converter's
            3. Possible parameter definition mismatch\
            '''.format(self.state.name, self.item)
        )


class ReadOnlyError(Exception):
    pass


class SendFailedError(epyqlib.utils.general.ExpectedException):
    def expected_message(self):
        return 'Send failed, make sure you are connected.'


class CanceledError(Exception):
    pass


@enum.unique
class State(enum.Enum):
    idle = 0
    reading = 1
    writing = 2


@enum.unique
class Priority(enum.IntEnum):
    user = 0
    background = 1


@attr.s
class Request:
    priority = attr.ib()
    read = attr.ib(cmp=False)
    meta = attr.ib(cmp=False)
    signals = attr.ib(cmp=False)
    deferred = attr.ib(cmp=False)
    passive = attr.ib(cmp=False)
    all_values = attr.ib(cmp=False)
    frame = attr.ib(cmp=False)
    send_time = attr.ib(default=None)


class Protocol(twisted.protocols.policies.TimeoutMixin):
    def __init__(self, timeout=1):
        self._deferred = None

        self._state = State.idle
        self._previous_state = self._state

        self._active = False

        self._request_memory = None
        self._timeout = timeout

        self.requests = queue.PriorityQueue()

        self.cancel_queued = False

    @property
    def state(self):
        return self._state

    @state.setter
    def state(self, new_state):
        logger.debug('Entering state {}'.format(new_state))
        self._previous_state = self._state
        self._state = new_state

    def makeConnection(self, transport):
        self._transport = transport
        logger.debug('Protocol.makeConnection(): {}'.format(transport))

    def _start_transaction(self):
        if self._active:
            raise Exception('Protocol is already active')

        self._active = True

    def _transaction_over(self):
        self.setTimeout(None)
        import twisted.internet
        twisted.internet.reactor.callLater(0, self._transaction_over_after_delay)
        d = self._deferred
        self._deferred = None
        self._request_memory = None
        self.state = State.idle
        return d

    def _transaction_over_after_delay(self):
        self._active = False
        self._get()

    def read(self, nv_signal, meta, priority=Priority.background, passive=False,
             all_values=False):
        return self._read_write_request(
            nv_signals=(nv_signal,),
            read=True,
            meta=meta,
            priority=priority,
            passive=passive,
            all_values=all_values,
        )

    def read_multiple(self, nv_signals, meta, priority=Priority.background,
                      passive=False, all_values=False):
        # TODO: make sure all signals are from the same frame
        return self._read_write_request(
            nv_signals=nv_signals,
            read=True,
            meta=meta,
            priority=priority,
            passive=passive,
            all_values=all_values,
        )

    def write(self, nv_signal, meta, priority=Priority.background,
              passive=False, ignore_read_only=False, all_values=False):
        # TODO: make sure all signals are from the same frame
        if nv_signal.frame.read_write.min > 0:
            if ignore_read_only:
                return
            else:
                raise ReadOnlyError()

        return self._read_write_request(
            nv_signals=(nv_signal,),
            read=False,
            meta=meta,
            priority=priority,
            passive=passive,
            all_values=all_values,
        )

    def write_multiple(self, nv_signals, meta, priority=Priority.background,
                       passive=False, ignore_read_only=False, all_values=False):
        if tuple(nv_signals)[0].frame.read_write.min > 0:
            if ignore_read_only:
                return
            else:
                raise ReadOnlyError()

        if not isinstance(nv_signals, dict):
            nv_signals = {
                s: (
                    s.value
                    if meta == epyqlib.nv.MetaEnum.value
                    else getattr(s.meta, meta.name).value
                )
                for s in nv_signals
            }

        return self._read_write_request(
            nv_signals=nv_signals,
            read=False,
            meta=meta,
            priority=priority,
            passive=passive,
            all_values=all_values,
        )

    def _read_write_request(self, nv_signals, read, meta, priority, passive,
                            all_values):
        deferred = twisted.internet.defer.Deferred()

        if not isinstance(nv_signals, dict):
            nv_signals = {
                s: (
                    s.value
                    if meta == epyqlib.nv.MetaEnum.value
                    else getattr(s.meta, meta.name).value
                )
                for s in nv_signals
            }

        frame = tuple(nv_signals.keys())[0].frame

        self._put(Request(
            read=read,
            meta=meta,
            signals=nv_signals,
            deferred=deferred,
            priority=priority,
            passive=passive,
            all_values=all_values,
            frame=frame,
        ))

        return deferred

    def _put(self, request):
        self.requests.put(request)
        self._get()

    def _get(self):
        if not self._active:
            while True:
                try:
                    request = self.requests.get(block=False)
                except queue.Empty:
                    self.cancel_queued = False
                else:
                    if self.cancel_queued:
                        request.deferred.errback(CanceledError())
                    elif request.read:
                        self._read_write(request)
                    else:
                        self._read_before_write(request)

                if not self.cancel_queued:
                    break

    def _read_before_write(self, request):
        if isinstance(request.signals, dict):
            nonskip = request.signals
        else:
            nonskip = {
                s: (
                    s.value
                    if request.meta == epyqlib.nv.MetaEnum.value
                    else getattr(s.meta, request.meta.name).value
                )
                for s in request.signals
            }

        skip_signals = (
            set(request.frame.parameter_signals) - set(nonskip.keys())
        )

        if len(skip_signals) == 0:
            try:
                self._read_write(request)
            except Exception as e:
                request.deferred.errback(e)
        else:
            d = twisted.internet.defer.Deferred()
            d.callback(None)

            def read_then_write(args, nonskip=nonskip):
                values, meta = args
                data = {k: v for k, v in nonskip.items()}
                for s, v in values.items():
                    s = s.set_signal
                    if s in request.frame.parameter_signals:
                        if s not in data:
                            data[s] = s.from_human(v)

                return self.write_multiple(
                    nv_signals=data,
                    meta=request.meta,
                    all_values=True,
                )

            def write_response(args, nonskip=nonskip, request=request):
                values, meta = args
                data = {
                    signal.status_signal: values[signal.status_signal]
                    for signal in nonskip
                }

                request.deferred.callback((data, request.meta))

            d.addCallback(lambda _: self.read_multiple(
                request.frame.parameter_signals,
                meta=request.meta,
                all_values=True
            ))
            d.addCallback(read_then_write)
            d.addCallback(write_response)
            d.addErrback(lambda e: request.deferred.errback(e))

    def _read_write(self, request):
        self._deferred = request.deferred
        self._start_transaction()
        self.state = State.reading if request.read else State.writing

        read_write, = (k for k, v
                       in request.frame.read_write.enumeration.items()
                       if v == ('Read' if request.read else 'Write'))

        data = collections.OrderedDict()
        for signal in request.frame.signals:
            if signal is request.frame.read_write:
                data[signal] = read_write
            elif signal.enumeration_name == 'Meta':
                data[signal] = request.meta.value
            elif signal not in request.frame.parameter_signals:
                data[signal] = signal.value
            elif signal in request.signals and not request.read:
                data[signal] = request.signals[signal]
            else:
                data[signal] = None

            if data[signal] is None:
                v = next(
                    v
                    for v in (
                        signal.default_value,
                        signal.from_human(signal.min),
                        signal.from_human(signal.max),
                        0,
                    )
                    if v is not None
                )
                data[signal] = v

            data[signal] = int(data[signal])

        data = request.frame.update_from_signals(
            data=data.values(),
            only_return=True,
        )

        if request.passive:
            write = self._transport.write_passive
        else:
            write = self._transport.write

        self._request_memory = request

        if not write(request.frame.to_message(data)):
            self.send_failed()
            return

        request.send_time = time.time()

        self.setTimeout(self._timeout)

    def dataReceived(self, msg):

        if not self._active:
            return

        if self._deferred is None:
            return

        request = self._request_memory
        if request is None:
            return

        if (
                not (
                    msg.arbitration_id == request.frame.status_frame.id
                    and (
                        bool(msg.is_extended_id)
                        == request.frame.status_frame.extended
                    )
                )
        ):
            return

        status_signal = tuple(request.signals)[0].status_signal

        if status_signal is None:
            return

        signals = status_signal.frame.unpack(msg.data, only_return=True)

        mux = status_signal.set_signal.frame.mux.value
        response_mux_value, = (
            v for k, v in signals.items()
            if k.name.endswith('_MUX')
        )
        if response_mux_value != mux:
            return
        meta_mux_value = tuple(
            v for k, v in signals.items()
            if k.enumeration_name == 'Meta'
        )
        if len(meta_mux_value) == 1:
            meta_mux_value, = meta_mux_value
            if meta_mux_value != request.meta.value:
                print(' -- skipping due to unmatched meta')
                return

        response_read_write_value = signals[status_signal.frame.command_signal]
        # TODO: handle the enumeration
        if response_read_write_value != request.read:
            return

        self.setTimeout(None)

        if request.all_values:
            status_signals = {s.status_signal for s in request.signals}
            value = {s: s.to_human(value=v) for s, v in signals.items()
                     if s in status_signals}
        else:
            raw_value = signals[status_signal]
            value = status_signal.to_human(value=raw_value)

        self.callback(value, request.meta)

    def send_failed(self):
        self.cancel_queued = True
        deferred = self._transaction_over()
        deferred.errback(SendFailedError())

    def timeoutConnection(self):
        request = self._request_memory
        # TODO: report all requested signals
        signal = tuple(request.signals)[0]
        mux_name = signal.frame.mux_name

        e = RequestTimeoutError(
            state=self.state,
            item=(
                f'{mux_name}:{signal.name} '
                f'({request.meta.name}, {request.send_time}, {time.time()}'
            ),
        )

        logger.debug(str(e))
        if self._previous_state in [State.idle]:
            self.state = self._previous_state
        deferred = self._transaction_over()
        deferred.errback(e)

    def callback(self, payload, meta):
        deferred = self._transaction_over()
        logger.debug('calling back for {}'.format(deferred))
        deferred.callback((payload, meta))

    def errback(self, payload):
        deferred = self._transaction_over()
        logger.debug('erring back for {}'.format(deferred))
        logger.debug('with payload {}'.format(payload))
        deferred.errback(payload)

    def cancel(self):
        deferred = self._transaction_over()
        deferred.cancel()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'listmenuview.ui'
#
# Created by: PyQt5 UI code generator 5.13.0
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(897, 802)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(Form)
        self.verticalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.esc_button = QtWidgets.QPushButton(Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.esc_button.sizePolicy().hasHeightForWidth())
        self.esc_button.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setFamily("FontAwesome")
        self.esc_button.setFont(font)
        self.esc_button.setObjectName("esc_button")
        self.horizontalLayout_2.addWidget(self.esc_button)
        self.label = QtWidgets.QLabel(Form)
        self.label.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.label.setObjectName("label")
        self.horizontalLayout_2.addWidget(self.label)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)
        self.line = QtWidgets.QFrame(Form)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.verticalLayout_2.addWidget(self.line)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.list_view = QtWidgets.QListView(Form)
        self.list_view.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.list_view.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.list_view.setUniformItemSizes(True)
        self.list_view.setObjectName("list_view")
        self.horizontalLayout.addWidget(self.list_view)
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.up_button = QtWidgets.QPushButton(Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.up_button.sizePolicy().hasHeightForWidth())
        self.up_button.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setFamily("FontAwesome")
        self.up_button.setFont(font)
        self.up_button.setObjectName("up_button")
        self.verticalLayout.addWidget(self.up_button)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem)
        self.down_button = QtWidgets.QPushButton(Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.down_button.sizePolicy().hasHeightForWidth())
        self.down_button.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setFamily("FontAwesome")
        self.down_button.setFont(font)
        self.down_button.setObjectName("down_button")
        self.verticalLayout.addWidget(self.down_button)
        self.horizontalLayout.addLayout(self.verticalLayout)
        self.verticalLayout_2.addLayout(self.horizontalLayout)

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Form"))
        self.esc_button.setText(_translate("Form", ""))
        self.label.setText(_translate("Form", "TextLabel"))
        self.up_button.setText(_translate("Form", ""))
        self.down_button.setText(_translate("Form", ""))
